/*globals require*/
require([
    'jquery',
    'testsuite/qunit',
    'testsuite/sinon',
    'ep',

    'de_epages/presentation/ui/uploadimage',
    '$ready!'
], function ($, Q, sinon, ep) {
    'use strict';

    var fixture = $('#qunit-fixture');

    // unity
    ep.config.unity = true;

    Q.module('presentationUiUploadImage');

    Q.test('Checking uploadimage\'s methods', 8, function () {
        $.each([
            '_create',
            '_setOption',
            '_showConfirmation',
            '_onUpload',
            '_onDeleteFile',
            '_deleteFileOnServer',
            '_setUploaderData',
            'destroy'
        ], function () {
            Q.ok(
                $.isFunction($.ui.presentationUiUploadImage.prototype[this]),
                this + ' is a method of the widget'
            );
        });
    });

    Q.test('Check HTML structure, visible options and additional action', 12, function () {

        var checkbox;

        fixture.presentationUiUploadImage({
            title: 'test title',
            subtitle: 'test subtitle',
            showPreview: true,
            showDeleteFile: true,
            scaleText: 'Scale this!',
            deleteText: 'Delete this!',
            additionalAction: {
                tAdditionalAction: 'buttonText',
                hide: false,
                callback: function(evt) {
                    Q.ok(
                        true,
                        'Additional action callback was executed.'
                    );
                }
            }
        });

        Q.ok(
            $('.ep-uiUploadImage').length > 0,
            'Found node epUiUploadImage'
        );

        Q.ok(
            $('.ep-uiUploadImage-Container').length > 0,
            'Found node .ep-uiUploadImage-Container'
        );

        Q.ok(
            $('.ep-uiUploadImage-Options').length > 0,
            'Found node .ep-uiUploadImage-Options'
        );

        Q.ok(
            $('.ep-uiUploadImage-Options .de_epages-presentationUiUploader-button').length > 0,
            'Found Uploader'
        );

        Q.strictEqual(
            $('.UploadImageConfirmation').css('display'),
            'none',
            'Confirmation test initially hidden'
        );

        checkbox = $('input[name=toggleScales]');

        Q.strictEqual(
            $('label[for=' + checkbox.attr('id') + ']').html(),
            'Scale this!',
            'Scale text is set corectly'
        );

        Q.ok(
            checkbox.prop('checked'),
            'Checkbox is initially checked'
        );

        Q.strictEqual(
            $('#deleteFile').html(),
            'Delete this!',
            'Delete text set correctly'
        );

        Q.ok(
            $('.ep-uploadimage-additional-btn').is(':visible'),
            'Additional button is visible'
        );

        Q.strictEqual(
            $('.ep-uploadimage-additional-btn').text(),
            'buttonText',
            'Additional button text is correct'
        );

        $('.ep-uploadimage-additional-btn').trigger('click');

        fixture.presentationUiUploadImage('destroy');

        Q.strictEqual(
            fixture.find('.ep-uiUploadImage').length,
            0,
            'Widget is destroyed'
        );
    });

    Q.test('Check HTML structure when options are set to be falsy and additional action is hidden', 4, function () {
        fixture.presentationUiUploadImage({
            title: 'test title',
            subtitle: 'test subtitle',
            showPreview: false,
            showDeleteFile: false,
            additionalAction: {
                tAdditionalAction: 'buttonText',
                hide: true,
                callback: function(evt) {}
            }
        });

        Q.ok(
            $('.ep-uiUploadImage-Container').length === 0,
            'Preview omitted'
        );

        Q.ok(
            $('input[name=toggleScales]').length === 0,
            'Scales omitted'
        );

        Q.ok(
            $('#deleteFile').parent().hasClass('HideElement'),
            'Delete button hidden'
        );

        Q.ok(
            $('.ep-uploadimage-additional-btn').hasClass('HideElement'),
            'Additional button is hidden'
        );

        fixture.presentationUiUploadImage('destroy');
    });

    Q.test('Testing server communication', function () {
        var xhr = sinon.useFakeXMLHttpRequest(),
            requests = sinon.requests = [];

        fixture.presentationUiUploadImage({
            title: 'test title',
            subtitle: 'test subtitle',
            showPreview: true,
            showDeleteFile: true,
            uploadData: {
                'ChangeAction': 'SaveImage'
            },
            deleteData: {
                'ViewAction': 'JSONViewResponse',
                'ObjectID': 17139,
                'ChangeAction': 'DeleteAllProductImages',
                'ACCEPT': 1
            }
        });

        xhr.onCreate = function (req) {
            requests.push(req);
        };

        // trigger delete file and confirm
        $('#deleteFile').trigger('click');
        $($('.ui-dialog-buttonset button')[0]).trigger('click');

        Q.strictEqual(
            requests[0].requestBody,
            'ViewAction=JSONViewResponse&ObjectID=17139&ChangeAction=DeleteAllProductImages&ACCEPT=1',
            'Check request for deleting fields from server'
        );

        fixture.presentationUiUploadImage('destroy');
    });
});